"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.cliOptionsTypes = exports.rcOptionsTypes = exports.commandNames = void 0;
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const rimraf_1 = __importDefault(require("@zkochan/rimraf"));
const execa_1 = __importDefault(require("execa"));
const render_help_1 = __importDefault(require("render-help"));
const makeEnv_1 = require("./makeEnv");
exports.commandNames = ['dlx'];
function rcOptionsTypes() {
    return {};
}
exports.rcOptionsTypes = rcOptionsTypes;
const cliOptionsTypes = () => ({
    package: [String, Array],
});
exports.cliOptionsTypes = cliOptionsTypes;
function help() {
    return (0, render_help_1.default)({
        description: 'Run a package in a temporary environment.',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'The package to install before running the command',
                        name: '--package',
                    },
                ],
            },
            common_cli_options_help_1.OUTPUT_OPTIONS,
        ],
        url: (0, cli_utils_1.docsUrl)('dlx'),
        usages: ['pnpm dlx <command> [args...]'],
    });
}
exports.help = help;
async function handler(opts, params) {
    var _a;
    const prefix = path_1.default.join(fs_1.default.realpathSync(os_1.default.tmpdir()), `dlx-${process.pid.toString()}`);
    const bins = process.platform === 'win32'
        ? prefix
        : path_1.default.join(prefix, 'bin');
    fs_1.default.mkdirSync(prefix, { recursive: true });
    process.on('exit', () => {
        try {
            fs_1.default.rmdirSync(prefix, {
                recursive: true,
                maxRetries: 3,
            });
        }
        catch (err) { }
    });
    await (0, rimraf_1.default)(bins);
    const pkgs = (_a = opts.package) !== null && _a !== void 0 ? _a : params.slice(0, 1);
    const pnpmArgs = ['add', ...pkgs, '--global', '--global-dir', prefix, '--dir', prefix];
    if (opts.reporter) {
        pnpmArgs.push(`--reporter=${opts.reporter}`);
    }
    await (0, execa_1.default)('pnpm', pnpmArgs, {
        stdio: 'inherit',
    });
    await (0, execa_1.default)(versionless(scopeless(params[0])), params.slice(1), {
        env: (0, makeEnv_1.makeEnv)({ userAgent: opts.userAgent, prependPaths: [bins] }),
        stdio: 'inherit',
    });
}
exports.handler = handler;
function scopeless(pkgName) {
    if (pkgName.startsWith('@')) {
        return pkgName.split('/')[1];
    }
    return pkgName;
}
function versionless(scopelessPkgName) {
    return scopelessPkgName.split('@')[0];
}
//# sourceMappingURL=dlx.js.map