"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.completion = exports.cliOptionsTypes = exports.rcOptionsTypes = exports.shorthands = exports.PARALLEL_OPTION_HELP = exports.IF_PRESENT_OPTION_HELP = exports.IF_PRESENT_OPTION = void 0;
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const config_1 = require("@pnpm/config");
const error_1 = __importDefault(require("@pnpm/error"));
const lifecycle_1 = __importStar(require("@pnpm/lifecycle"));
const pick_1 = __importDefault(require("ramda/src/pick"));
const realpath_missing_1 = __importDefault(require("realpath-missing"));
const render_help_1 = __importDefault(require("render-help"));
const runRecursive_1 = __importDefault(require("./runRecursive"));
const existsInDir_1 = __importDefault(require("./existsInDir"));
const exec_1 = require("./exec");
exports.IF_PRESENT_OPTION = {
    'if-present': Boolean,
};
exports.IF_PRESENT_OPTION_HELP = {
    description: 'Avoid exiting with a non-zero exit code when the script is undefined',
    name: '--if-present',
};
exports.PARALLEL_OPTION_HELP = {
    description: 'Completely disregard concurrency and topological sorting, \
running a given script immediately in all matching packages \
with prefixed streaming output. This is the preferred flag \
for long-running processes such as watch run over many packages.',
    name: '--parallel',
};
exports.shorthands = {
    parallel: [
        '--workspace-concurrency=Infinity',
        '--no-sort',
        '--stream',
        '--recursive',
    ],
};
function rcOptionsTypes() {
    return {
        ...(0, pick_1.default)([
            'npm-path',
        ], config_1.types),
    };
}
exports.rcOptionsTypes = rcOptionsTypes;
function cliOptionsTypes() {
    return {
        ...(0, pick_1.default)([
            'bail',
            'sort',
            'unsafe-perm',
            'workspace-concurrency',
            'scripts-prepend-node-path',
        ], config_1.types),
        ...exports.IF_PRESENT_OPTION,
        recursive: Boolean,
        reverse: Boolean,
    };
}
exports.cliOptionsTypes = cliOptionsTypes;
const completion = async (cliOpts, params) => {
    var _a, _b;
    if (params.length > 0) {
        return [];
    }
    const manifest = await (0, cli_utils_1.readProjectManifestOnly)((_a = cliOpts.dir) !== null && _a !== void 0 ? _a : process.cwd(), cliOpts);
    return Object.keys((_b = manifest.scripts) !== null && _b !== void 0 ? _b : {}).map((name) => ({ name }));
};
exports.completion = completion;
exports.commandNames = ['run', 'run-script'];
function help() {
    return (0, render_help_1.default)({
        aliases: ['run-script'],
        description: 'Runs a defined package script.',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Run the defined package script in every package found in subdirectories \
or every workspace package, when executed inside a workspace. \
For options that may be used with `-r`, see "pnpm help recursive"',
                        name: '--recursive',
                        shortAlias: '-r',
                    },
                    {
                        description: 'The command will exit with a 0 exit code even if the script fails',
                        name: '--no-bail',
                    },
                    exports.IF_PRESENT_OPTION_HELP,
                    exports.PARALLEL_OPTION_HELP,
                    ...common_cli_options_help_1.UNIVERSAL_OPTIONS,
                ],
            },
            common_cli_options_help_1.FILTERING,
        ],
        url: (0, cli_utils_1.docsUrl)('run'),
        usages: ['pnpm run <command> [-- <args>...]'],
    });
}
exports.help = help;
async function handler(opts, params) {
    var _a, _b, _c, _d, _e;
    let dir;
    const [scriptName, ...passedThruArgs] = params;
    if (opts.recursive) {
        if (scriptName || Object.keys(opts.selectedProjectsGraph).length > 1) {
            return (0, runRecursive_1.default)(params, opts);
        }
        dir = Object.keys(opts.selectedProjectsGraph)[0];
    }
    else {
        dir = opts.dir;
    }
    const manifest = await (0, cli_utils_1.readProjectManifestOnly)(dir, opts);
    if (!scriptName) {
        const rootManifest = opts.workspaceDir && opts.workspaceDir !== dir
            ? (await (0, cli_utils_1.tryReadProjectManifest)(opts.workspaceDir, opts)).manifest
            : undefined;
        return printProjectCommands(manifest, rootManifest !== null && rootManifest !== void 0 ? rootManifest : undefined);
    }
    if (scriptName !== 'start' && !((_a = manifest.scripts) === null || _a === void 0 ? void 0 : _a[scriptName])) {
        if (opts.ifPresent)
            return;
        if (opts.fallbackCommandUsed) {
            if (opts.argv == null)
                throw new Error('Could not fallback because opts.argv.original was not passed to the script runner');
            return (0, exec_1.handler)({
                selectedProjectsGraph: {},
                ...opts,
            }, opts.argv.original.slice(1));
        }
        if (opts.workspaceDir) {
            const { manifest: rootManifest } = await (0, cli_utils_1.tryReadProjectManifest)(opts.workspaceDir, opts);
            if ((_b = rootManifest === null || rootManifest === void 0 ? void 0 : rootManifest.scripts) === null || _b === void 0 ? void 0 : _b[scriptName]) {
                throw new error_1.default('NO_SCRIPT', `Missing script: ${scriptName}`, {
                    hint: `But ${scriptName} is present in the root of the workspace,
so you may run "pnpm -w run ${scriptName}"`,
                });
            }
        }
        throw new error_1.default('NO_SCRIPT', `Missing script: ${scriptName}`);
    }
    const lifecycleOpts = {
        depPath: dir,
        extraBinPaths: opts.extraBinPaths,
        pkgRoot: dir,
        rawConfig: opts.rawConfig,
        rootModulesDir: await (0, realpath_missing_1.default)(path_1.default.join(dir, 'node_modules')),
        scriptsPrependNodePath: opts.scriptsPrependNodePath,
        scriptShell: opts.scriptShell,
        silent: opts.reporter === 'silent',
        shellEmulator: opts.shellEmulator,
        stdio: 'inherit',
        unsafePerm: true, // when running scripts explicitly, assume that they're trusted.
    };
    const existsPnp = existsInDir_1.default.bind(null, '.pnp.cjs');
    const pnpPath = (_c = (opts.workspaceDir && await existsPnp(opts.workspaceDir))) !== null && _c !== void 0 ? _c : await existsPnp(dir);
    if (pnpPath) {
        lifecycleOpts.extraEnv = (0, lifecycle_1.makeNodeRequireOption)(pnpPath);
    }
    try {
        if (opts.enablePrePostScripts &&
            ((_d = manifest.scripts) === null || _d === void 0 ? void 0 : _d[`pre${scriptName}`]) &&
            !manifest.scripts[scriptName].includes(`pre${scriptName}`)) {
            await (0, lifecycle_1.default)(`pre${scriptName}`, manifest, lifecycleOpts);
        }
        await (0, lifecycle_1.default)(scriptName, manifest, { ...lifecycleOpts, args: passedThruArgs });
        if (opts.enablePrePostScripts &&
            ((_e = manifest.scripts) === null || _e === void 0 ? void 0 : _e[`post${scriptName}`]) &&
            !manifest.scripts[scriptName].includes(`post${scriptName}`)) {
            await (0, lifecycle_1.default)(`post${scriptName}`, manifest, lifecycleOpts);
        }
    }
    catch (err) { // eslint-disable-line
        if (opts.bail !== false) {
            throw err;
        }
    }
    return undefined;
}
exports.handler = handler;
const ALL_LIFECYCLE_SCRIPTS = new Set([
    'prepublish',
    'prepare',
    'prepublishOnly',
    'prepack',
    'postpack',
    'publish',
    'postpublish',
    'preinstall',
    'install',
    'postinstall',
    'preuninstall',
    'uninstall',
    'postuninstall',
    'preversion',
    'version',
    'postversion',
    'pretest',
    'test',
    'posttest',
    'prestop',
    'stop',
    'poststop',
    'prestart',
    'start',
    'poststart',
    'prerestart',
    'restart',
    'postrestart',
    'preshrinkwrap',
    'shrinkwrap',
    'postshrinkwrap',
]);
function printProjectCommands(manifest, rootManifest) {
    var _a;
    const lifecycleScripts = [];
    const otherScripts = [];
    for (const [scriptName, script] of Object.entries((_a = manifest.scripts) !== null && _a !== void 0 ? _a : {})) {
        if (ALL_LIFECYCLE_SCRIPTS.has(scriptName)) {
            lifecycleScripts.push([scriptName, script]);
        }
        else {
            otherScripts.push([scriptName, script]);
        }
    }
    if (lifecycleScripts.length === 0 && otherScripts.length === 0) {
        return 'There are no scripts specified.';
    }
    let output = '';
    if (lifecycleScripts.length > 0) {
        output += `Lifecycle scripts:\n${renderCommands(lifecycleScripts)}`;
    }
    if (otherScripts.length > 0) {
        if (output !== '')
            output += '\n\n';
        output += `Commands available via "pnpm run":\n${renderCommands(otherScripts)}`;
    }
    if ((rootManifest === null || rootManifest === void 0 ? void 0 : rootManifest.scripts) == null) {
        return output;
    }
    const rootScripts = Object.entries(rootManifest.scripts);
    if (rootScripts.length === 0) {
        return output;
    }
    if (output !== '')
        output += '\n\n';
    output += `Commands of the root workspace project (to run them, use "pnpm -w run"):
${renderCommands(rootScripts)}`;
    return output;
}
function renderCommands(commands) {
    return commands.map(([scriptName, script]) => `  ${scriptName}\n    ${script}`).join('\n');
}
//# sourceMappingURL=run.js.map